/**
 * PROJECT:  LASER DEMO
 * AUTHOR:  danpost (greenfoot username)
 * CREATED:  January 27, 2017
 * 
 * DESCRIPTION:  a scenario with a laser box that emits a laser beam controlled by the spacebar
 * and four reflectors to alter the path of the beam; one reflector can be rotated by left and
 * right arrow keys
 */
import greenfoot.*;

public class MyWorld extends World
{
    LaserBox laserBox; // object that emits laser beams
    Actor reflector; // the one that rotates
    int keyTimer; // to regulate speed of rotation of reflector
    
    /** constructs the world */
    public MyWorld()
    {
        super(400, 400, 1);
        setPaintOrder(LaserBox.class);
        buildBackground();
        introduceActors();
    }
    
    /** internal method to change background color and provide instructions */
    private void buildBackground()
    {
        GreenfootImage bg = getBackground();
        bg.setColor(Color.BLACK);
        bg.fill();
        String text = "Press space to emit beam";
        GreenfootImage img = new GreenfootImage(text, 23, Color.CYAN, new Color(0, 0, 0, 0));
        bg.drawImage(img, 200-img.getWidth()/2, 375);
        text = "Use left and right arrows to turn first reflector";
        img = new GreenfootImage(text, 23, Color.CYAN, new Color(0, 0, 0, 0));
        bg.drawImage(img, 200-img.getWidth()/2, 0);
    }
    
    /** internal method to add objects into world */
    private void introduceActors()
    {
        addObject(laserBox = new LaserBox(0), 157, 148);
        addObject(reflector = new Reflector(90), 355, 145);
        addObject(new Reflector(216), 131, 305);
        addObject(new Reflector(0), 220, 54);
        addObject(new Reflector(144), 306, 302);
        addObject(new Reflector(288), 86, 149);
    }
    
    /** controls the rotational change of the first reflector hit by beam */
    public void act()
    {
        // the timer that regulates rotational speed of reflector
        if (keyTimer > 0 && --keyTimer != 0) return;
        // the requested change
        int dr = 0;
        if (Greenfoot.isKeyDown("left")) dr--;
        if (Greenfoot.isKeyDown("right")) dr++;
        // the change
        if (dr != 0) 
        {
            reflector.turn(dr); // turn reflector
            // all beams hitting and reflecting off this reflector must be refreshed
            for (Object obj : getObjects(Beam.class)) ((Beam)obj).cutOffSource(reflector);
            keyTimer = 150; // restart timer
        }
    }
}